Extend

Enum Extend 

#[repr(u8)]
pub enum Extend { Pad = 0, Repeat = 1, Reflect = 2, }
Defines how a brush is extended when the content does not fill a shape.

Variants



Pad = 0

Extends the image by repeating the edge color of the brush.



Repeat = 1

Extends the image by repeating the brush.



Reflect = 2

Extends the image by reflecting the brush.