pub fn try_pattern(
steps: impl IntoIterator<Item = LinetypeStep>,
) -> Result<Arc<[LinetypeStep]>, PatternError>Expand description
pattern but hands back the reason instead of panicking.
pub fn try_pattern(
steps: impl IntoIterator<Item = LinetypeStep>,
) -> Result<Arc<[LinetypeStep]>, PatternError>pattern but hands back the reason instead of panicking.